//========================================================================================
//
//	File:				DrawFrm.h
//	Release Version:	$ 1.0d11 $
//
//	Author:				Henri Lamiraux
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef DRAWFRM_H
#define DRAWFRM_H

// ----- Part Layer -----

#ifndef FWFRMING_H
#include "FWFrming.h"
#endif

//========================================================================================
// Forward Declaration
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma import on
#endif
class FW_CLASS_ATTR FW_CPresentation;
class FW_CLASS_ATTR FW_CGrowBox;
#if FW_LIB_EXPORT_PRAGMAS
#pragma import off
#endif

class FW_CLASS_ATTR CDrawPart;
class FW_CLASS_ATTR CDrawSelection;
class FW_CLASS_ATTR FW_CRuler;

//========================================================================================
// CDrawFrame
//========================================================================================

class FW_CLASS_ATTR CDrawFrame : public FW_CEmbeddingFrame
{
public:
	FW_DECLARE_CLASS

//----------------------------------------------------------------------------------------
// Initialization/destruction
//
public:		
	CDrawFrame(Environment *ev, ODFrame* odFrame, FW_CPresentation* presentation, CDrawPart* drawPart);
	virtual ~ CDrawFrame();

//----------------------------------------------------------------------------------------
// Inherited API
//
public:		
	// ----- Menu
	virtual FW_Boolean		DoAdjustMenus(Environment *ev, FW_CMenuBar* menuBar, FW_Boolean hasMenuFocus, FW_Boolean isRoot);
	virtual FW_Boolean		DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent);
	
	// ----- Events -----
	virtual FW_Boolean		DoActivateEvent(Environment* ev, const FW_CActivateEvent& theActivateEvent);

	// ----- Embedding -----
	virtual void			EmbedSingleFrame(Environment* ev, 
											ODPart* odPart, 
											ODFrame* odEmbeddedFrame, 
											ODShape* frameShape);

	virtual ODFacet*		CreateEmbeddedFacet(Environment* ev,
											ODFacet* embeddingFacet,
											FW_MProxy* proxy,
											ODFrame* embeddedFrame,
											ODShape* proposedClipShape);
	
	// ----- Geometry -----									
	virtual void			GeometryChanged(Environment *ev,
											ODFacet* odFacet,
											FW_Boolean clipShapeChanged,
											FW_Boolean externalTransformChanged);
	virtual void			FrameShapeChanged(Environment *ev);
											
	// ----- Window Size ------
	virtual void			AdjustZoomedWindowSize(Environment* ev, FW_CPoint& proposedSize);
	virtual void			AdjustWindowGrowLimits(Environment* ev, FW_CPoint& minSize, FW_CPoint& maxSize);
	
	// ----- Gadget Layout -----
	virtual void				CreateSubViews(Environment* ev);
	virtual void				AdjustSubViews(Environment* ev);

	// ----- Commands -----
	virtual FW_CEditCommand* 	NewEditCommand(Environment *ev, ODCommandID commandID);
	virtual FW_CDragCommand* 	NewDragCommand(Environment *ev, FW_CFrame* theFrame);
	virtual FW_CDropCommand*	NewDropCommand(Environment *ev, FW_CFrame* frame,
												ODDragItemIterator* dropInfo, 
												ODFacet* facet, const FW_CPoint& windowPoint);
												
	// ----- Printing ------
	virtual FW_CPrintHandler*	NewPrintHandler(Environment* ev);
	virtual	FW_Boolean			IsCurrentlyPrintable(Environment* ev) const;
	virtual void				GetPrintContentExtent(Environment *ev, FW_CPoint& extent) const;

	// ----- Drawing ------
	virtual void 				Draw(Environment *ev, ODFacet* odFacet, ODShape* invalidShape);

	// ----- Miscellaneous -----
	virtual void				FacetAdded(Environment* ev, ODFacet* facet);

	virtual FW_Boolean			RevealFrame(Environment *ev, 
										ODFrame* embeddedFrame, 
										ODShape* revealShape);

	// ----- Internalize/Externalize -----
	virtual void			ExternalizeFrame(Environment* ev, ODStorageUnitView* storageUnitView);
	virtual void			InternalizeFrame(Environment* ev, ODStorageUnitView* storageUnitView);
	
//----------------------------------------------------------------------------------------
//	New API
//
public:
	ODShape*				AcquireContentShape(Environment *ev, ODShape* invalidShape);
	CDrawSelection*			GetSelection(Environment *ev);
	void					ToggleRulers(Environment *ev, FW_Boolean visible);

	FW_Boolean				IsGraphicsGridOn() const;
	FW_Boolean				AreRulersOn() const;
	
//----------------------------------------------------------------------------------------
// Data Members
//
private:	
	CDrawPart*				fDrawPart;
	FW_CGrowBox*			fGrowBox;
	CDrawSelection*			fSelection;
	
	FW_CRuler*				fHorzRuler;
	FW_CRuler*				fVertRuler;
	
	FW_Boolean				fGridShown;
	FW_Boolean				fRulersShown;
};

//----------------------------------------------------------------------------------------
//	CDrawFrame::GetSelection
//----------------------------------------------------------------------------------------
inline CDrawSelection*	CDrawFrame::GetSelection(Environment *ev)
{
	return fSelection;
}

//----------------------------------------------------------------------------------------
//	CDrawFrame::IsGraphicsGridOn
//----------------------------------------------------------------------------------------
inline FW_Boolean CDrawFrame::IsGraphicsGridOn() const
{
	return fGridShown;
}

//----------------------------------------------------------------------------------------
//	CDrawFrame::AreRulersOn
//----------------------------------------------------------------------------------------
inline FW_Boolean CDrawFrame::AreRulersOn() const
{
	return fRulersShown;
}

#endif
