//========================================================================================
//
//	File:				LineShp.h
//	Release Version:	$ 1.0d11 $
//
//	Author:				Henri Lamiraux
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef LINESHP_H
#define LINESHP_H

#ifndef CONSTANT_H
#include "Constant.h"
#endif

#ifndef BASESHP_H
#include "BaseShp.h"
#endif

// ----- OS Layer -----

#ifndef FWPOINT_H
#include "FWPoint.h"
#endif

#ifndef FWRECT_H
#include "FWRect.h"
#endif

// ----- Foundation Layer -----

#ifndef FWRUNTYP_H
#include "FWRunTyp.h"
#endif

//========================================================================================
// Forward Declarations
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma import on
#endif
/*class FW_CLASS_ATTR ODFacet;
class FW_CLASS_ATTR ODShape;
class FW_CLASS_ATTR CDrawPart;
class FW_CLASS_ATTR CDrawPublishLink;
class FW_CLASS_ATTR CDrawSubscribeLink;
class FW_CLASS_ATTR FW_CGraphicContext;
class FW_CLASS_ATTR FW_CMouseEvent;
class FW_CLASS_ATTR FW_CRectShape;*/
#if FW_LIB_EXPORT_PRAGMAS
#pragma import off
#endif

//class FW_CLASS_ATTR CDrawFacetClipper;

//========================================================================================
// class CLineShape
//========================================================================================

class FW_CLASS_ATTR CLineShape : public CBaseShape
{
public:
	FW_DECLARE_CLASS

public:
	CLineShape();
	CLineShape(FW_CReadableStream& archive);
	virtual ~CLineShape();
		
	// ----- Geometry
	virtual void		SetShapeGeometry(const FW_CPoint& anchorPoint,  const FW_CPoint& currentPoint);
	virtual ODShape*	CreateShapeOutline(Environment *ev);
	virtual void		GetClipRegion(Environment* ev, ODShape* clipRegion);
	virtual void		GetRectGeometry(FW_CRect& bounds) const;
	
	FW_CPoint			GetLineStart() const
							{return fStart;}
	FW_CPoint			GetLineEnd() const
							{return fEnd;}
							
	// ----- Selection
	virtual FW_Boolean	HitTest(Environment *ev, FW_CGraphicContext& gc, const FW_CMouseEvent& theMouseEvent) const;
	
	// ----- Dragging
	virtual void		OffsetShape(Environment *ev, FW_CFixed xDelta, FW_CFixed yDelta);

	// ----- Resize
	virtual void		ResizeFeedback(FW_CGraphicContext& gc, const FW_PInk& ink, const FW_PStyle& style, 
										short whichHandle, const FW_CPoint& mouseLoc);
	virtual void		GetMapRects(short whichHandle, const FW_CPoint& lastLocation,
									FW_CRect& srcRect, FW_CRect& dstRect);
	virtual void		GetHandleCenter(short whichHandle, FW_CPoint& center) const;
	virtual void		MapShape(Environment *ev, CDrawPart* part, const FW_CRect& srcRect, const FW_CRect& dstRect);
	
	// ----- Drawing
	virtual void 		RenderShape(Environment* ev, ODFacet* facet, FW_CGraphicContext& gc);

	// ----- Persistence
	virtual void 		Flatten(FW_CWritableStream& archive);
	
	// ----- Archiving -----
	static void* 		Read(FW_CReadableStream& archive);

private:
	void 				OutlineShape(FW_CGraphicContext& gc, const FW_PInk& ink, const FW_PStyle& style, 
									 const FW_CPoint& pt1, const FW_CPoint& pt2);

private:
	FW_CPoint			fStart;
	FW_CPoint			fEnd;
};

#endif