//========================================================================================
//
//	File:				EmbedCmd.h
//	Release Version:	$ 1.0d11 $
//
//	Author:				John Wendt
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef EMBEDCMD_H
#define EMBEDCMD_H

// ----- Part Layer -----

#ifndef FWEDCMD_H
#include "FWEdCmd.h"
#endif

#ifndef FWDRCMD_H
#include "FWDrCmd.h"
#endif

//========================================================================================
// Forward Declarations
//========================================================================================

class FW_CLASS_ATTR CEmbedPart;
class FW_CLASS_ATTR CEmbedFrame;
class FW_CLASS_ATTR CEmbedSelection;
class FW_CLASS_ATTR CEmbedProxy;

//========================================================================================
// CEmbedEditCommand
//========================================================================================

class FW_CLASS_ATTR CEmbedEditCommand : public FW_CEditCommand
{
  public:

	CEmbedEditCommand(Environment *ev, 
					  ODCommandID commandID,
					  CEmbedPart* part, 
					  FW_CFrame* frame, 
					  FW_CSelection* selection,
					  FW_Boolean canUndo);

	virtual ~CEmbedEditCommand();

	// --- FW_CCommand overrides
	virtual void UndoIt(Environment *ev);			// Override
	virtual void RedoIt(Environment *ev);			// Override
	virtual void SaveUndoState(Environment *ev);	// Override
	virtual void SaveRedoState(Environment *ev);	// Override
	virtual void FreeUndoState(Environment *ev);	// Override
	virtual void FreeRedoState(Environment *ev);	// Override

  private:
	void RemovePart(Environment* ev);
	void RestorePart(Environment* ev);
	void RestoreOldPart(Environment* ev);

  private:
	CEmbedPart*			fEmbedPart;
	CEmbedProxy*		fPastedProxy;	// saved proxy for Redo
	CEmbedProxy*		fOldProxy;		// saved proxy for Undo
};

//========================================================================================
//	class CProxyDropCommand
//========================================================================================

class FW_CLASS_ATTR CProxyDropCommand : public FW_CDropCommand
{
  public:
	CProxyDropCommand(Environment *ev,
					  CEmbedPart* itsPart,
					  FW_CFrame* frame,
					  ODDragItemIterator* dropInfo, 
					  ODFacet* odFacet,
					  const FW_CPoint& dropPoint);

	virtual ~ CProxyDropCommand();

	// --- for Undo/Redo ---
	virtual void		UndoIt(Environment *ev);	// Override
	virtual void		RedoIt(Environment *ev);	// Override
	virtual void		CommitDone(Environment *ev);	// Override
	virtual void		CommitUndone(Environment *ev);	// Override
	virtual void		SaveUndoState(Environment *ev);	// Override
	virtual void		SaveRedoState(Environment *ev);	// Override

  private:
	CEmbedPart*			fEmbedPart;
	CEmbedSelection*	fEmbedSelection;
	CEmbedProxy*		fDroppedProxy;	// saved proxy for Redo
	CEmbedProxy*		fOldProxy;		// saved proxy for Undo
};


#endif
