//========================================================================================
//
//	File:				FormPrt.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FORMPRT_H
#define FORMPRT_H

#ifndef FORMDEF_H
#include "FormDef.h"
#endif

// ----- Framework Includes -----

#ifndef FWPART_H
#include "FWPart.h"
#endif

// ----- OS Layer -----

#ifndef FWRESFIL_H
#include "FWResFil.h"
#endif

// ----- Foundation Layer -----

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWBNDSTR_H
#include "FWBndStr.h"
#endif


//========================================================================================
// Forward Declarations
//========================================================================================

class FW_CLASS_ATTR FW_CPart;
class FW_CLASS_ATTR FW_CString;
class FW_CLASS_ATTR FW_CMenuBar;
class FW_CLASS_ATTR FW_CMenuEvent;
class FW_CLASS_ATTR FW_CPresentation;
class FW_CLASS_ATTR FW_CPictureShape;

//========================================================================================
// CFormPart
//========================================================================================

class FW_CLASS_ATTR CFormPart : public FW_CPart
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	CFormPart(ODPart* odPart);
	virtual ~CFormPart();

	virtual void Initialize(Environment* ev);	// Override

//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual FW_CFrame*	NewFrame(Environment* ev,
								 		ODFrame* odFrame,
								 		FW_CPresentation* presentation,
								 		FW_Boolean fromStorage);

	virtual FW_Boolean	DoAdjustMenus(Environment* ev,
									 	FW_CMenuBar* menuBar, 
									  	FW_Boolean hasMenuFocus,
									  	FW_Boolean isRoot);

	virtual FW_Boolean	DoMenu(Environment* ev,
							   			const FW_CMenuEvent& theMenuEvent);

	virtual void		ExternalizeContent(Environment *ev,
										ODStorageUnit* storageUnit,
										FW_CCloneInfo* cloneInfo);
									
	virtual void		InternalizeContent(Environment *ev,
										ODStorageUnit* storageUnit, 
										FW_CCloneInfo* cloneInfo);

//----------------------------------------------------------------------------------------
//	New API
//
public:
	const FW_CString& 	GetTextData();
	void				InitPicture(Environment* ev);	
	FW_CPictureShape* 	GetPictShape();
	FW_CPictureShape* 	GetPictAdorner();
	FW_CPoint			GetPictSize();
	
	void 				DoFormCommand(Environment* ev, ODCommandID id);

	void 				GetResourceString(FW_ResourceId multiStringId,
						   				FW_ResourceId stringId,
						   				FW_CString& string);

//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	static const ODValueType kPartKind;
	static const ODValueType kPartUserName;

	FW_CString255			fTextData;
	FW_Boolean				fIsForm;
	FW_CPresentation*		fPresentation;
	FW_CPictureShape*		fPictShape;	
	FW_CPictureShape*		fPictAdorner;	
	FW_CPoint				fPictSize;		
};

//==============================================================================
// CFormPart Inlines
//==============================================================================

inline FW_CPictureShape* CFormPart::GetPictShape()
{
	return fPictShape;
}

inline FW_CPoint CFormPart::GetPictSize()
{
	return fPictSize;
}

inline FW_CPictureShape* CFormPart::GetPictAdorner()
{
	return fPictAdorner;
}


#endif
