//========================================================================================
//
//	File:				FormPrt.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "Form.hpp"

#ifndef FORMPRT_H
#include "FormPrt.h"
#endif

#ifndef FORMFRM_H
#include "FormFrm.h"
#endif

#ifndef FORMDEF_H
#include "FormDef.h"
#endif

// ----- Framework Layer -----

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWITERS_H
#include "FWIters.h"
#endif

#ifndef FWPRESEN_H
#include "FWPresen.h"
#endif

// ----- OS Layer -----

#ifndef FWMENU_H
#include "FWMenu.h"
#endif

#ifndef FWCFMRES_H
#include "FWCFMRes.h"
#endif

#ifndef FWRESTYP_H
#include "FWResTyp.h"
#endif

#ifndef FWSUSINK_H
#include "FWSUSink.h"
#endif

#ifndef FWEVENT_H
#include "FWEvent.h"
#endif

#ifndef FWPICSHP_H
#include <FWPicShp.h>			
#endif

// ----- Foundation Layer -----

#ifndef FWSTREAM_H
#include "FWStream.h"
#endif

#ifndef FWSTRS_H
#include "FWStrs.h"
#endif

#ifndef FWSTRGAR_H
#include "FWStrgAr.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

//========================================================================================
// Constants
//========================================================================================

const ODValueType CFormPart::kPartKind = kFormKind;
const ODValueType CFormPart::kPartUserName = kFormEditorUserString;

const ODCommandID cFormCommand		= FW_kFirstUserCommandID;
const ODCommandID cGoodbyeCommand		= cFormCommand + 1;

#define kMainPresentation				"ODF:Presentation:Form"

//========================================================================================
//	Runtime info
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment odfform
#endif

//========================================================================================
//	CFormPart class
//========================================================================================

//----------------------------------------------------------------------------------------
// CFormPart constructor
//----------------------------------------------------------------------------------------

CFormPart::CFormPart(ODPart* odPart) :
	FW_CPart(odPart, CFormPart::kPartKind, CFormPart::kPartUserName, FW_gInstance, kPartIconID),
	fTextData(""),
	fIsForm(TRUE),
	fPresentation(NULL),
	fPictShape(NULL),
	fPictAdorner(NULL),
	fPictSize(FW_kZeroPoint)
{
}

//----------------------------------------------------------------------------------------
// CFormPart destructor
//----------------------------------------------------------------------------------------

CFormPart::~CFormPart()
{
	if (fPictShape)		delete fPictShape; 
	if (fPictAdorner)	delete fPictAdorner; 
}

//----------------------------------------------------------------------------------------
// CFormPart::Initialize
//----------------------------------------------------------------------------------------

void CFormPart::Initialize(Environment* ev)	// Override
{
	FW_CPart::Initialize(ev);

	// ----- Register our Presentation
	fPresentation = RegisterPresentation(ev, kMainPresentation, TRUE, NULL);	// no selection

	// ----- Initialize the Streamr component -----
	FW_InitializeArchiving(FW_gInstance);

	// ----- Open the resource file for this shared library -----
	FW_CSharedLibraryResourceFile resFile;

	// ----- Create the Form menu -----
	FW_CPullDownMenu* pullDownMenu = new FW_CPullDownMenu(ev, resFile, kFormPartStrings, kFormMenuString);

	// ----- Add the Form menu items -----
	pullDownMenu->AppendTextItem(ev, resFile, kFormPartStrings, kGreetMenuString, cFormCommand, '1');
	pullDownMenu->AppendTextItem(ev, resFile, kFormPartStrings, kGoodbyeMenuString, cGoodbyeCommand, '2');

	// ----- Add the Form menu to the menu bar -----
	GetMenuBar(ev)->AdoptMenuLast(ev, pullDownMenu);

	// Initialize the text data string
	FW_CString32 menuString;
	::FW_LoadStringByID(resFile, kFormPartStrings, MULTISTRINGRES, kFormString, fTextData);
	::FW_LoadStringByID(resFile, kFormPartStrings, MULTISTRINGRES, kPlatformString, menuString);
	fTextData += menuString;
	
	// Initialize pictures
	InitPicture(ev);
}

//----------------------------------------------------------------------------------------
// CFormPart::InitPicture
//----------------------------------------------------------------------------------------
void CFormPart::InitPicture(Environment* ev)
{
	// Open the resource file for this shared library 
	FW_CSharedLibraryResourceFile resFile;	
	
#ifdef FW_BUILD_MAC
	const short kFormPictID = 130;
	FW_PPicture formPict(resFile, kFormPictID);
	FW_CRect bounds;
	formPict->GetPictBounds(bounds);
	fPictShape = FW_NEW(FW_CPictureShape, (formPict, bounds));
	fPictSize = bounds.Size();

	const short kAdornPictID = 131;
	FW_PPicture buttonsPict(resFile, kAdornPictID);
	buttonsPict->GetPictBounds(bounds);
	bounds.Offset(FW_IntToFixed(50), FW_IntToFixed(5));
	fPictAdorner = FW_NEW(FW_CPictureShape, (buttonsPict, bounds));
#endif
}

//------------------------------------------------------------------------------
// CFormPart::InternalizeContent
//------------------------------------------------------------------------------

void CFormPart::InternalizeContent(Environment* ev,
									ODStorageUnit* storageUnit, 
									FW_CCloneInfo* cloneInfo)	// Override
{
	FW_UNUSED(cloneInfo);

	if (FW_SUExistsThenFocus(ev, storageUnit, kODPropContents, CFormPart::kPartKind))
	{
		if (storageUnit->GetSize(ev) != 0)
		{
			// Read the text data string from storage
			FW_CStorageUnitSink suSink(storageUnit, kODPropContents, CFormPart::kPartKind);
			FW_CReadableStream stream(&suSink);
			
			stream >> fTextData;
			stream >> fIsForm;
		}
	}
}

//----------------------------------------------------------------------------------------
// CFormPart::ExternalizeContent
//----------------------------------------------------------------------------------------

void CFormPart::ExternalizeContent(Environment* ev,
									ODStorageUnit* storageUnit,
									FW_CCloneInfo* cloneInfo)	// Override
{
	FW_UNUSED(cloneInfo);

	storageUnit->Focus(ev, kODPropContents, kODPosUndefined, CFormPart::kPartKind, 0, kODPosUndefined);
	storageUnit->Remove(ev);
	storageUnit->AddValue(ev, CFormPart::kPartKind);

	// Write the text data string to storage
	FW_CStorageUnitSink suSink(storageUnit, kODPropContents, CFormPart::kPartKind);
	FW_CWritableStream stream(&suSink);
	
	stream << fTextData;
	stream << fIsForm;
}

//----------------------------------------------------------------------------------------
// CFormPart::NewFrame
//----------------------------------------------------------------------------------------

FW_CFrame* CFormPart::NewFrame(Environment* ev,
								ODFrame* odFrame,
								FW_CPresentation* presentation,
								FW_Boolean fromStorage)	// Override
{
	FW_UNUSED(fromStorage);

	return new CFormFrame(ev, odFrame, presentation, this);
}

//----------------------------------------------------------------------------------------
// CFormPart::DoMenu
//----------------------------------------------------------------------------------------

FW_Boolean CFormPart::DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent) // Override
{
	FW_Boolean menuHandled = TRUE;
	ODCommandID id = theMenuEvent.GetCommandID(ev);
	
	switch (id)
	{
		case cFormCommand:
		case cGoodbyeCommand:
			this->DoFormCommand(ev, id);
			break;

		default:
			menuHandled = FALSE;
	}
	
	return menuHandled;
}

//----------------------------------------------------------------------------------------
// CFormPart::DoAdjustMenus
//----------------------------------------------------------------------------------------

FW_Boolean CFormPart::DoAdjustMenus(Environment* ev, FW_CMenuBar* menuBar, 
									 FW_Boolean hasMenuFocus,
									 FW_Boolean isRoot)	// Override
{
	if (hasMenuFocus)
	{	
		// Set up the Form menu
		menuBar->EnableCommand(ev, cFormCommand, !fIsForm);
		menuBar->EnableCommand(ev, cGoodbyeCommand, fIsForm);
	}
	
	return FALSE;
}

//----------------------------------------------------------------------------------------
// CFormPart::DoFormCommand
//----------------------------------------------------------------------------------------

void CFormPart::DoFormCommand(Environment* ev, ODCommandID id)
{
	FW_CString32 string;

	this->GetResourceString(kFormPartStrings, kPlatformString, string);

	if (id == cFormCommand)
	{
		this->GetResourceString(kFormPartStrings, kFormString, fTextData);
	}
	else if (id == cGoodbyeCommand)
	{
		this->GetResourceString(kFormPartStrings, kGoodbyeString, fTextData);
	}
	fTextData += string;

	// Toggle the form flag
	fIsForm = !fIsForm;

	// Mark the document as changed so it can be saved
	this->Changed(ev);

	// Mark the display frame as changed, so that containing parts can update links
	FW_CPresentationFrameIterator piter(fPresentation);
	FW_CFrame* frame = piter.First();
	this->ContentUpdated(ev, frame, kODUnknownUpdate);

	// Force all frames to be redrawn
	fPresentation->Invalidate(ev);
}

//----------------------------------------------------------------------------------------
// CFormPart::GetResourceString
//----------------------------------------------------------------------------------------

void CFormPart::GetResourceString(FW_ResourceId multiStringId,
								   FW_ResourceId stringId,
								   FW_CString& string)
{
	// ----- Open the resource file for this shared library -----
	FW_CSharedLibraryResourceFile resFile;

	// ----- Read the string from the resource file -----
	::FW_LoadStringByID(resFile, multiStringId, MULTISTRINGRES, stringId, string);
}

//----------------------------------------------------------------------------------------
// CFormPart::GetTextData
//----------------------------------------------------------------------------------------

const FW_CString& CFormPart::GetTextData()
{
	return fTextData;
}

